# -*- coding: utf-8 -*-
from django.conf import settings

PRICE_ATTR = getattr(settings, 'BASKET_PRICE_ATTR', 'price')
DEFAULT_BASKET_FORM = 'basket.forms.DefaultOrderForm'

BASKET_FORM = getattr(settings, 'BASKET_FORM', DEFAULT_BASKET_FORM)

BASKET_OPTIONS_USE_KEEP = getattr(settings, 'BASKET_OPTIONS_USE_KEEP', True)
BASKET_OPTIONS_USE_DELETE = getattr(settings, 'BASKET_OPTIONS_USE_DELETE', False)



# -*- coding: utf-8 -*-
from django import forms
from models import OrderInfo

class OrderInfoForm(forms.ModelForm):
    class Meta:
        model = OrderInfo
        exclude = ['order']

    def __init__(self, request, *args, **kwds):
        kwds.update({
            'instance': OrderInfo(order=request.order),
        })
        return super(OrderInfoForm, self).__init__(*args, **kwds)



from classytags.core import Tag, Options
from classytags.arguments import Argument
from classytags.helpers import InclusionTag
from django import template
from django.contrib.contenttypes.models import ContentType
from basket.models import *

register = template.Library()

class AddBasketButton(InclusionTag):
    name = 'add_basket_button'
    template = 'basket/button.html'
    
    options = Options(
        Argument('object', required=True, resolve=True),
        Argument('button_text', required=True, resolve=False),
        Argument('added_text', required=True, resolve=False),
        Argument('already_in_text', required=True, resolve=False),
        Argument('template_name', required=False, resolve=False),
    )

    def get_template(self, context, object, button_text, added_text, already_in_text, template_name):
        if template_name:
            template_to_render = template_name
        else:
            template_to_render = self.template
        return template_to_render
    
    def get_context(self, context, object, button_text, added_text, already_in_text, template_name):
        context_to_return = {}
        if object is None:
            context_to_return.update({
                'content_type': None,
                'object': None,
            })
        else:
            content_type = ContentType.objects.get_for_model(object)
            
            try:
                order_id = context['request'].session['order_id']
            
                # quantity of such items those are already into basket
                already_in_qty = BasketItem.objects.filter(order=order_id, content_type=content_type, object_id=object.id).count()
            
                # is any number of such items in the basket
                already_in = bool(already_in_qty)
            except KeyError:
                # for private mode
                already_in = False
                
            context_to_return.update({
                'content_type': content_type,
                'object': object,
                'button_text': button_text,
                'added_text': added_text,
                'already_in': already_in,
                'already_in_text': already_in_text,
            })
        return context_to_return

register.tag(AddBasketButton)

@register.filter
def content_type(object):
    '''
    returns content type id for object
    '''
    content_type = ContentType.objects.get_for_model(object)
    return '%s' % content_type.id



# -*- coding: utf-8 -*-
from basket.models import Order
from basket.signals import order_submit
from basket import settings as basket_settings
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

class OrderInfo(models.Model):
    '''
    Order information model. If you want to override default fields, set BASKET_MODEL in settings.py.
    For more information check default settings in basket/settings.py
    '''
    class Meta:
        verbose_name = _('Order information')
        verbose_name_plural = _('Order information')

    order = models.OneToOneField(Order)
    registered = models.DateTimeField(verbose_name=_('Order creation date and time'),
        auto_now_add=True)
    name = models.CharField(verbose_name=_('Customer name'), max_length=100)
    address = models.CharField(verbose_name=_('Address'), max_length=100)
    city = models.CharField(verbose_name=_('City'), max_length=50)
    telephone = models.CharField(verbose_name=_('Phone number'), max_length=100)

    email = models.EmailField(verbose_name=_('Email'), max_length=100,
        blank=True, null=True)
    comment = models.CharField(verbose_name=_('Comment'), max_length=200,
        blank=True, null=True)


def add_orderinfo(sender, **kwargs):
    order = kwargs['order']
    cleaned_data = kwargs['data']
    cleaned_data.update({
        'order': order,
    })
    OrderInfo.objects.create(**cleaned_data)

order_submit.connect(add_orderinfo, sender=Order)

if basket_settings.BASKET_FORM == basket_settings.DEFAULT_BASKET_FORM:
    basket_settings.BASKET_FORM = 'basket.contrib.orderinfo.forms.OrderInfoForm'



# -*- coding: utf-8 -*-
from django.contrib import admin
from django import forms
from basket.models import Order
from basket.contrib.orderinfo.models import OrderInfo
#from django.utils.translation import ugettext_lazy as _


class InfoInline(admin.StackedInline):
    extra = 0
    max_num = 1
    can_delete = False
    model = OrderInfo


class OrderInfoAdmin(admin.ModelAdmin):
    inlines = [InfoInline]
    exclude = ['user', 'session_key', ]
    list_display = ['__unicode__', 'goods', 'summary', 'get_status', 'registered', 'user']
    fieldsets = None
    search_fields = ('user__username',)


try:
    admin.site.unregister(Order)
    admin.site.register(Order, OrderInfoAdmin)
except admin.sites.AlreadyRegistered:
    pass



# -*- coding: utf-8 -*-
from basket.forms import OrderFormset
from basket.models import Order
from basket.utils import render_to, get_order_form, send_mail
from basket.signals import order_submit
from basket.settings import BASKET_OPTIONS_USE_KEEP, BASKET_OPTIONS_USE_DELETE
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.views.decorators.http import require_http_methods
from django.http import HttpResponseBadRequest, HttpResponseRedirect, Http404
from django.template import loader


@render_to('basket/basket.html')
def basket(request):
    # do not create order automatically
    order = request.order
    if order is None:
        return HttpResponseRedirect(reverse('basket-empty'))

    if request.method == 'POST':
        formset = OrderFormset(request.POST, instance=order)

        if formset.is_valid():
            formset.save()

            if 'refresh' in request.POST:
                return HttpResponseRedirect(reverse('basket'))
            else:
                return HttpResponseRedirect(reverse('order_confirm'))
    else:
        formset = OrderFormset(instance=order)

    return {
        'order_form': get_order_form()(request),
        'formset': formset,
        'order': order,
        'keep': BASKET_OPTIONS_USE_KEEP,
        'delete': BASKET_OPTIONS_USE_DELETE,
    }

@render_to('basket/confirm.html')
def confirm(request):
    # do not create order automatically
    order = request.order

    if order is None or order.empty():
        return HttpResponseRedirect(reverse('basket-empty'))

    if request.method == 'POST':
        form = get_order_form()(request, request.POST)
        if form.is_valid():
            order_submit.send(sender=Order, order=order, data=form.cleaned_data)
            return HttpResponseRedirect(reverse('basket-thankyou'))
    else:
        form = get_order_form()(request)
    return {
        'form': form,
        'order': order,
        'keep': BASKET_OPTIONS_USE_KEEP,
        'delete': BASKET_OPTIONS_USE_DELETE,
    }


def thankyou(request):
    '''Use generic view instead'''
    raise NotImplementedError

def status(request, order_id=None):
    '''Use generic view instead'''
    raise NotImplementedError

@require_http_methods(["POST"])
@render_to('basket/summary.html')
def add_to_basket(request):
    # Automatically create order if it does not exist
    if request.order is None:
        order = Order.from_request(request)
        request.order = order
    else:
        order = request.order

    content_type_id = request.REQUEST.get('content_type', None)
    object_id = request.REQUEST.get('object_id', None)
    try:
        content_type = ContentType.objects.get(id=content_type_id)
        item = content_type.get_object_for_this_type(id=object_id)
    except ObjectDoesNotExist:
        raise Http404

    order.add_item(item)
    return {}


@require_http_methods(["POST"])
@render_to('basket/summary.html')
def delete_from_basket(request):
    # Do not automatically create order
    order = request.order
    if order is None:
        raise Http404(_('Basket does not exist'))

    content_type_id = request.REQUEST.get('content_type', None)
    object_id = request.REQUEST.get('object_id', None)
    try:
        content_type = ContentType.objects.get(id=content_type_id)
        item = content_type.get_object_for_this_type(id=object_id)
    except ObjectDoesNotExist:
        raise Http404

    order.remove_item(item)
    return {'order': order}



# -*- coding: utf-8 -*-
from basket.settings import PRICE_ATTR
from basket.signals import order_submit
from basket.utils import get_order_form, send_mail
from datetime import datetime
from decimal import Decimal
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.contrib.sessions.models import Session
from django.db.models import Count
from django.db import models
from django.template import loader
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext

STATUS_PENDING = 0
STATUS_NEW = 1
STATUS_PROCESS = 2
STATUS_CLOSED = 3
STATUS_ERROR = 4

STATUS_CHIOCES = (
    (STATUS_PENDING, _('Pending')),
    (STATUS_NEW, _('New')),
    (STATUS_PROCESS, _('Process')),
    (STATUS_CLOSED, _('Closed & OK')),
    (STATUS_ERROR, _('Closed with error')),
)

def query_set_factory(model_name, query_set_class):
    class ChainedManager(models.Manager):

        def get_query_set(self):
            model = models.get_model('basket', model_name)
            return query_set_class(model)

        def __getattr__(self, attr, *args):
            try:
                return getattr(self.__class__, attr, *args)
            except AttributeError:
                return getattr(self.get_query_set(), attr, *args)
    return ChainedManager()

class OrderQuerySet(models.query.QuerySet):

    def active_orders(self):
        '''Filters active orders, which can be changed by user'''
        return self.filter(status=STATUS_PENDING)


class Order(models.Model):
    class Meta:
        verbose_name = _('Order')
        verbose_name_plural = _('Orders')

    user = models.ForeignKey(User, verbose_name=_('User'), null=True, blank=True)
    session_key = models.CharField(verbose_name=_('Session key'),
        max_length=40, null=True, blank=True)
    # fields from order status
    status = models.IntegerField(verbose_name=_('Order status'), choices=STATUS_CHIOCES)
    created = models.DateTimeField(verbose_name=_('Created date'))
    comment = models.TextField(verbose_name=_('Comment'), blank=True, null=True)

    objects = query_set_factory('Order', OrderQuerySet)

    def registered(self):
        '''
        Returns True is order is from registered user
        '''
        if self.user is None:
            return False
        else:
            return True
    registered.short_description = _('Registered user')
    registered.boolean = True

    @classmethod
    def from_request(cls, request):
        order = cls()
        if request.user.is_authenticated():
            order.user = request.user
        else:
            order.session_key = request.session.session_key
        order.status = STATUS_PENDING
        order.created = datetime.now()
        order.save()
        request.session['order_id'] = order.id
        return order

    def add_item(self, item):
        item_ct = ContentType.objects.get_for_model(item)
        already_in_order = bool(
            self.items.filter(object_id=item.id, content_type=item_ct).count()
        )

        if already_in_order:
            basket_item = self.items.get(object_id=item.id, content_type=item_ct)
            basket_item.quantity += 1
            basket_item.save()
        else:
            basket_item = BasketItem(content_object=item, quantity=1, order=self)
            basket_item.save()
            self.items.add(basket_item)
            self.save()

    def remove_item(self, item):
        item_ct = ContentType.objects.get_for_model(item)
        self.items.filter(object_id=item.id, content_type=item_ct).delete()

    def set_quantity(self, item, quantity):
        item_ct = ContentType.objects.get_for_model(item)
        already_in_order = bool(
            self.items.filter(object_id=item.id, content_type=item_ct).count()
        )

        if quantity <= 0:
            if already_in_order:
                self.remove_item(item)
            return

        if already_in_order:
            basket_item = self.items.get(object_id=item.id, content_type=item_ct)
            basket_item.quantity = quantity
            basket_item.save()
        else:
            basket_item = BasketItem(content_object=item, quantity=quantity, order=self)
            basket_item.save()
            self.items.add(basket_item)
            self.save()

    def flush(self):
        for basket_item in self.items.all():
            basket_item.delete()

    def calculate(self):
        total_goods = 0
        total_price = Decimal('0.0')
        for basket_item in self.items.all():
            total_goods += basket_item.quantity
            try:
                total_price += basket_item.get_sum()
            except AttributeError:
                pass
            except TypeError:
                total_price = _('Undefined')
        return {'goods': total_goods, 'summary': total_price}

    def goods(self):
        return self.calculate()['goods']
    goods.short_description = _('Total items in basket')

    def summary(self):
        return self.calculate()['summary']
    summary.short_description = _('Total price')

    def get_status(self):
        return self.status_set.latest('modified')
    get_status.short_description = _('Order status')

    def empty(self):
        return self.goods() == 0

    def __unicode__(self):
        return 'order #%s' % self.id


class BasketItem(models.Model):
    class Meta:
        ordering = ['object_id']

    order = models.ForeignKey('Order', related_name='items', null=True, blank=False)

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    quantity = models.IntegerField(verbose_name=_('Quantity'))

    def get_price(self):
        if callable(PRICE_ATTR):
            value = PRICE_ATTR(self.content_object)
        else:
            value = getattr(self.content_object, PRICE_ATTR)
        if callable(value):
            value = value()
        if value is None:
            return _('Undefined')
        return value

    def get_sum(self):
        p = self.get_price()
        q = self.quantity
        if p is None:
            return _('Undefined')
        return p * q 


def comment_order(order, form_data):
    OrderForm = get_order_form()
    result = {}
    for field_name, value in form_data.iteritems():
        result.update({
            field_name: (value, OrderForm.base_fields[field_name].label),
        })

    message = loader.render_to_string('basket/order.txt', {
        'order': order,
        'form_data': result,
    })
    return message

def email_to_managers(sender, **kwargs):
    '''Send email when order issued'''
    managers = [manager[1] for manager in settings.MANAGERS]
    subject = _('New order from site')

    order = kwargs['order']
    form_data = kwargs['data']
    message = comment_order(order, form_data)
    send_mail(subject, message, managers)

def change_status(sender, **kwargs):
    order = kwargs['order']
    order.comment = ugettext('Automatically created status')
    order.status = STATUS_NEW
    order.save()

def autocomment(sender, **kwargs):
    order = kwargs['order']
    form_data = kwargs['data']
    order.comment = '\n'.join((
        '%s' % order.comment,
        comment_order(order, form_data),
        ugettext('New order'),
    ))
    order.save()

order_submit.connect(email_to_managers, sender=Order)
order_submit.connect(change_status, sender=Order)
order_submit.connect(autocomment, sender=Order)
